#include <string.h>
#include "stm32f0xx.h"

#include "stm32f0_usb.h"
#include "usbreq.h"
#include "usbdesc.c"

#include "translate.h"

volatile uint8_t ep0_state=0;
volatile uint8_t ep1_wait=0;
uint8_t usb_address;

STD_Request *ep0_std_req = (STD_Request *)(USB_PMA+128);

char *kbd_report = (char *)(USB_PMA+192);

char setup_packet_service(void);
char descriptor_service(void);
char ep0_preparedata(void *ptr, int16_t len);

void show_LED(uint8_t);

volatile char scan_row=0;
unsigned char prev_key_state[14];
unsigned char key_state[14];        // key states (bit line: 0 is ON, 1 is OFF)

void update_key_matrix(char row, char col, char onoff);

const uint32_t GPIOA_DEFAULT =GPIO_MODER_MODER14_1|GPIO_MODER_MODER13_1; // PA14, PA13 AF0 (SWD)
const uint32_t GPIOB_DEFAULT =GPIO_MODER_MODER13_0|GPIO_MODER_MODER12_0 // User1, User2 output
                   |GPIO_MODER_MODER11_1|GPIO_MODER_MODER10_1   // PB11, PB10 AF4 (USART3)
                   |GPIO_MODER_MODER2_0;                        // PB2 LED output
const uint32_t GPIOC_DEFAULT =GPIO_MODER_MODER15_0;     // PC15 output (CLK to 74LS174)

int main(void)
{
    int i;

    RCC->AHBENR |= RCC_AHBENR_GPIOAEN|RCC_AHBENR_GPIOBEN|RCC_AHBENR_GPIOCEN;  //|RCC_AHBENR_GPIOFEN;    // enable GPIO ports clock

    GPIOA->MODER = GPIOA_DEFAULT;
    GPIOB->MODER = GPIOB_DEFAULT;
    GPIOB->AFR[1] = 0x00004400; // AF4 for PB11,PB10 : USART3 RX,TX 
    GPIOC->MODER = GPIOC_DEFAULT;

    GPIOA->PUPDR = 0x00005555;  // pull-up for key scan (PA0..7)

    RCC->CR2 |= RCC_CR2_HSI48ON;
    while(!(RCC->CR2 & RCC_CR2_HSI48RDY));  // wait until HSI48 ready
    RCC->CFGR2 = RCC_CFGR2_PREDIV1_DIV8;
    RCC->CFGR = RCC_CFGR_PLLSRC_HSI48_PREDIV|RCC_CFGR_PLLXTPRE|RCC_CFGR_PLLMUL2;    // PLL src: HSI48/8*2=12MHz
    RCC->CR = RCC_CR_PLLON|RCC_CR_HSION;
    while(!(RCC->CR & RCC_CR_PLLRDY));  // wait until PLL clock ready
    RCC->CFGR = RCC_CFGR_PLLSRC_HSI48_PREDIV|RCC_CFGR_PLLXTPRE|RCC_CFGR_PLLMUL2|RCC_CFGR_SW_PLL;    // switch sysclk to PLL (12MHz)
    RCC->CR = RCC_CR_PLLON;     // HSI off

    RCC->APB1ENR |= RCC_APB1ENR_USBEN;
    NVIC_EnableIRQ(USB_IRQn);
    USB->CNTR = USB_CNTR_CTRM|USB_CNTR_PMAOVRM|USB_CNTR_ERRM|USB_CNTR_WKUPM
                |USB_CNTR_SUSPM|USB_CNTR_RESETM|USB_CNTR_SOFM; // leave power down mode
    USB->BCDR = USB_BCDR_DPPU;  // pull-up resistor enable

    for(i=0;i<14;i++)
    {
        key_state[i]=0xff;
        prev_key_state[i]=0xff;
    }

    RCC->APB1ENR |= RCC_APB1ENR_TIM6EN;     // enable basic timer 6
    TIM6->PSC = 999;    // prescaler
    TIM6->ARR = 5;      // auto reload value, interrupt frequency 2kHz
    TIM6->DIER = TIM_DIER_UIE;  // update interrput
    TIM6->CR1 = TIM_CR1_URS|TIM_CR1_CEN;    // start counter

    NVIC_EnableIRQ(TIM6_DAC_IRQn);

    while(1)
    {
        static char row=0;
        __WFI();
        if(ep0_state & 0x80)    // request data processing
        {
            if(ep0_state==0x80) // SETUP phase
            {
                if(!setup_packet_service())
                {
                    ep0_state=0;
                    // not supported
                }
                // ep0_state should be set to 1 or 2, if processed
            }
            else    // OUT phase
            {
                // process data
                show_LED(*(uint8_t *)(USB_PMA+128));
                ep0_state=4;
                USB_PMA[1]=0;       // Zero length DATA0
                USB->EP0R = USB_EP_TYPE_CONTROL|USB_EP_STAT_TX0;
            }
        }
        else
        {
            if(usb_address && ep0_state==0)
            {
                USB->DADDR = USB_DADDR_EF|usb_address;
                usb_address=0;
            }
        }
        if(row!=scan_row)   // new scan line
        {
            if(key_state[row]!=prev_key_state[row])
            {
                uint8_t test=0x80;
                uint8_t diff=key_state[row]^prev_key_state[row];
                for(i=0;i<8;i++)
                {
                    if(diff & test)
                        update_key_matrix(row,i,key_state[row]&test);
                    test>>=1;
                }
            }
            row=scan_row;
        }
    }
}

void TIM6_DAC_IRQHandler(void)
{
    __IO uint8_t *PA_IDR = (uint8_t *)&(GPIOA->IDR);

    TIM6->SR &= ~TIM_SR_UIF;
    prev_key_state[scan_row]=key_state[scan_row];
    key_state[scan_row]= *PA_IDR;   // update key states
    switch(scan_row)
    {
        case 13: // next row PB14
                GPIOC->MODER = GPIOC_DEFAULT;
                GPIOB->MODER = GPIOB_DEFAULT|GPIO_MODER_MODER14_0;
                break;
        case  0: // next row PB15
                GPIOB->MODER = GPIOB_DEFAULT|GPIO_MODER_MODER15_0;
                break;
        case  1: // next row PB3
                GPIOB->MODER = GPIOB_DEFAULT|GPIO_MODER_MODER3_0;
                break;
        case  2: // next row PB4
                GPIOB->MODER = GPIOB_DEFAULT|GPIO_MODER_MODER4_0;
                break;
        case  3: // next row PB5
                GPIOB->MODER = GPIOB_DEFAULT|GPIO_MODER_MODER5_0;
                break;
        case  4: // next row PB6
                GPIOB->MODER = GPIOB_DEFAULT|GPIO_MODER_MODER6_0;
                break;
        case  5: // next row PB7
                GPIOB->MODER = GPIOB_DEFAULT|GPIO_MODER_MODER7_0;
                break;
        case  6: // next row PB8
                GPIOB->MODER = GPIOB_DEFAULT|GPIO_MODER_MODER8_0;
                break;
        case  7: // next row PB9
                GPIOB->MODER = GPIOB_DEFAULT|GPIO_MODER_MODER9_0;
                break;
        case  8: // next row PA8
                GPIOB->MODER = GPIOB_DEFAULT;
                GPIOA->MODER = GPIOA_DEFAULT|GPIO_MODER_MODER8_0;
                break;
        case  9: // next row PA9
                GPIOA->MODER = GPIOA_DEFAULT|GPIO_MODER_MODER9_0;
                break;
        case 10: // next row PA10
                GPIOA->MODER = GPIOA_DEFAULT|GPIO_MODER_MODER10_0;
                break;
        case 11: // next row PA15
                GPIOA->MODER = GPIOA_DEFAULT|GPIO_MODER_MODER15_0;
                break;
        case 12: // next row PC13
                GPIOA->MODER = GPIOA_DEFAULT;
                GPIOC->MODER = GPIOC_DEFAULT|GPIO_MODER_MODER13_0;
                break;
    }
    if(scan_row<13)
        scan_row++;
    else
        scan_row=0;
}


void USB_IRQHandler(void)
{
    if(USB->ISTR & USB_ISTR_CTR)
    {
        if((USB->ISTR & 0x0f)==0)   // EP_ID==0
        {
            switch(ep0_state)
            {
                case 0: ep0_state |= 0x80;
                        break;
                case 1: if(USB->EP0R & USB_EP_CTR_TX)
                        {
                            USB->EP0R = USB_EP_TYPE_CONTROL|USB_EP_STAT_RX0;
                            ep0_state=3;
                            return;
                        }
                        else
                            ep0_state=0;
                        break;
                case 2: if((USB->EP0R & USB_EP_SETUP)==0)   // OUT packet
                            ep0_state |= 0x80;
                        else
                            ep0_state=0;
                        break;
                case 3: ep0_state=0;
                        break;
                case 4: ep0_state=0;
                        break;
                default:ep0_state=0;
                        break;
            }
            USB->EP0R = USB_EP_TYPE_CONTROL; // just clear flag
            return;
        }
        else    // EP_ID can be 1
        {
            USB->EP1R = USB_EP_TYPE_INTERRUPT|1; // just clear flag
            ep1_wait=0;
            return;
        }
    }
    if(USB->ISTR & USB_ISTR_PMAOVR)
    {
        USB->ISTR = ~USB_ISTR_PMAOVR;
    }
    if(USB->ISTR & USB_ISTR_ERR)
    {
        USB->ISTR = ~USB_ISTR_ERR;  // write 0 to clear
    }
    if(USB->ISTR & USB_ISTR_WKUP)
    {
        USB->CNTR &= ~USB_CNTR_FSUSP;   // no suspend
        USB->ISTR = ~USB_ISTR_WKUP; // write 0 to clear
    }
    if(USB->ISTR & USB_ISTR_SUSP)
    {
        USB->CNTR |= USB_CNTR_FSUSP;    // force suspend
        USB->CNTR |= USB_CNTR_LPMODE;   // low power
        USB->ISTR = ~USB_ISTR_SUSP; // write 0 to clear
    }
    if(USB->ISTR & USB_ISTR_RESET)
    {
        USB->BTABLE = 0;    // buffer table at bottom of PMA
        USB_PMA[0]=128; //ADDR0_TX
        USB_PMA[1]=0;   //COUNT0_TX
        USB_PMA[2]=256; //ADDR0_RX
        USB_PMA[3]=0x8000|(3<<10);  //RX buffer 128 bytes
        USB->EP0R = USB_EP_TYPE_CONTROL|USB_EP_STAT_RX1;
        ep0_state=0;
        USB_PMA[4]=384; //ADDR1_TX
        USB_PMA[5]=0;   //COUNT1_TX
        USB->EP1R = USB_EP_TYPE_INTERRUPT|1;    // EP1 type
        ep1_wait=0;
        USB->DADDR = USB_DADDR_EF;      // enable function
        USB->ISTR = ~USB_ISTR_RESET;    // write 0 to clear
    }
    if(USB->ISTR & USB_ISTR_SOF)
    {
        USB->ISTR = ~USB_ISTR_SOF;  // write 0 to clear
    }
}


char ep0_preparedata(void *ptr, int16_t len)
{
    int i;
    for(i=0;i<(len+1)>>1;i++)   // 16-bit access
        USB_PMA[64+i] = *((uint16_t *)ptr+i);
    USB_PMA[1]=len;
    USB->EP0R = USB_EP_TYPE_CONTROL|USB_EP_STAT_TX0;
    ep0_state=1;
    return 1;
}


char setup_packet_service(void)
{
    if(ep0_std_req->bmRequestType & 0x20)   // class-specific
    {
        switch(ep0_std_req->bRequest)
        {
            case REQ_GET_REPORT: break;
            case REQ_GET_IDLE:
                USB_PMA[64]=0xfa;   // return 1 byte
                USB_PMA[1]=1;
                USB->EP0R = USB_EP_TYPE_CONTROL|USB_EP_STAT_TX0;
                ep0_state=1;
                return 1;
                break;
            case REQ_SET_REPORT:
                USB->EP0R = USB_EP_TYPE_CONTROL|USB_EP_STAT_RX0;
                ep0_state=2;
                return 1;
                break;
            case REQ_SET_IDLE:
                USB_PMA[1]=0;   // Zero DATA
                USB->EP0R = USB_EP_TYPE_CONTROL|USB_EP_STAT_TX0;
                ep0_state=4;
                return 1;
                break;
        }
        return 0;
    }
    else    // standard
    {
        switch(ep0_std_req->bRequest)
        {
            case REQ_GET_DESCRIPTOR:
                return descriptor_service();
                break;
            case REQ_SET_ADDRESS:
                if(ep0_std_req->bmRequestType!=0x00)
                    return 0;
                usb_address=ep0_std_req->wValue;
                USB->EP0R = USB_EP_TYPE_CONTROL|USB_EP_STAT_TX0;
                USB_PMA[1]=0;       // Zero length DATA0
                ep0_state=4;    // No Data phase
                return 1;
            case REQ_SET_CONFIGURATION:
                if(ep0_std_req->bmRequestType!=0x00)
                    return 0;
                USB->EP1R = USB_EP_TYPE_INTERRUPT|USB_EP_STAT_TX1|1;
                USB->EP0R = USB_EP_TYPE_CONTROL|USB_EP_STAT_TX0;
                USB_PMA[1]=0;   // Zero DATA
                ep0_state=4;    // No DATA phase
                return 1;
            default: return 0;
        }
    }
}


char descriptor_service(void)
{
    switch((ep0_std_req->wValue)>>8)
    {
        case DESC_TYPE_DEVICE:
            return ep0_preparedata(&DevDesc, sizeof(DevDesc));
            break;
        case DESC_TYPE_CONFIG:
            if(sizeof(ConfigDescData)>ep0_std_req->wLength)
                return ep0_preparedata(&ConfigDescData, ep0_std_req->wLength);
            else
                return ep0_preparedata(&ConfigDescData, sizeof(ConfigDescData));
            break;
        case DESC_TYPE_STRING:
            switch(ep0_std_req->wValue &0xff)
            {
                case 0 : return ep0_preparedata(&LangIDDesc, sizeof(LangIDDesc));
                case 1 : return ep0_preparedata(&UnicodeVendorDesc, sizeof(UnicodeVendorDesc));
                case 2 : return ep0_preparedata(&UnicodeProdDesc, sizeof(UnicodeProdDesc));
                default: return 0;
            }
            break;
        case REPORT_DESC_TYPE:
            return ep0_preparedata(&HidReportDesc, sizeof(HidReportDesc));
        default:
            return 0;
    }
}


void show_LED(uint8_t status)
{
    __IO uint8_t *PA_MODER = (uint8_t *)&(GPIOA->MODER);
    __IO uint8_t *PA_ODR = (uint8_t *)&(GPIOA->ODR);
    uint8_t led=0xff;
    if(status & 1)
        led &= 0xfd;
    if(status & 2)
        led &= 0xfb;
    if(status & 4)
        led &= 0xfe;
    GPIOC->BRR = (1<<15);
    PA_MODER[0] = 0x15;
    PA_ODR[0] = led;
    GPIOC->BSRR = (1<<15);
    PA_MODER[0] = 0x00;
}

const char hid_keymap_qwerty[14][8]={
    {HK_RShift, HK_NONE, HK_NONE, HK_A, HK_R, HK_7, HK_F9, HK_Esc},
    {HK_F12, HK_BS, HK_Up, HKR_Slash, HKR_Aster, HK_NONE, HKR_Enter, HK_NONE},
    {HK_Period, HK_V, HK_BSlash, HK_J, HK_P, HK_Q, HK_4, HK_F6},
    {HK_Comma, HK_C, HK_Quote, HK_H, HK_O, HK_Equal, HK_3, HK_F5},
    {HK_M, HK_X, HK_Colon, HK_G, HK_I, HK_Minus, HK_2, HK_F4},
    {HK_LShift, HK_NONE, HK_Caps, HK_F, HK_U, HK_0, HK_1, HK_F3},
    {HK_Scroll, HK_LAlt, HK_RCtrl, HK_D, HK_Y, HK_9, HK_BQuote, HK_F2},
    {HK_NONE, HK_RAlt, HK_LCtrl, HK_S, HK_T, HK_8, HK_Enter, HK_F1},
    {HK_PgUp, HK_Home, HK_Insert, HKR_5, HKR_6, HKR_4, HK_F10, HKR_Dot},
    {HK_Right, HK_Down, HK_Left, HKR_Num, HKR_Plus, HK_NONE, HK_NONE, HK_NONE},
    {HK_PgDn, HK_End, HK_F11, HKR_9, HKR_Minus, HKR_8, HKR_7, HK_NONE},
    {HK_Space, HK_N, HK_Z, HK_L, HK_RBr, HK_E, HK_6, HK_F8},
    {HK_Slash, HK_B, HK_Delete, HK_K, HK_LBr, HK_W, HK_5, HK_F7},
    {HK_Pause, HK_Tab, HK_PrtScr, HKR_2, HKR_3, HKR_1, HKR_0, HK_MODE}
};

const char hid_keymap_dvorak[14][8]={
    {HK_RShift, HK_NONE, HK_NONE, HK_A, HK_P, HK_7, HK_F9, HK_Esc},
    {HK_F12, HK_BS, HK_Up, HKR_Slash, HKR_Aster, HK_NONE, HKR_Enter, HK_NONE},
    {HK_V, HK_K, HK_BSlash, HK_H, HK_L, HK_Quote, HK_4, HK_F6},
    {HK_W, HK_J, HK_Minus, HK_D, HK_R, HK_RBr, HK_3, HK_F5},
    {HK_M, HK_Q, HK_S, HK_I, HK_C, HK_LBr, HK_2, HK_F4},
    {HK_LShift, HK_NONE, HK_Caps, HK_U, HK_G, HK_0, HK_1, HK_F3},
    {HK_Scroll, HK_LAlt, HK_RCtrl, HK_E, HK_F, HK_9, HK_BQuote, HK_F2},
    {HK_NONE, HK_RAlt, HK_LCtrl, HK_O, HK_Y, HK_8, HK_Enter, HK_F1},
    {HK_PgUp, HK_Home, HK_Insert, HKR_5, HKR_6, HKR_4, HK_F10, HKR_Dot},
    {HK_Right, HK_Down, HK_Left, HKR_Num, HKR_Plus, HK_NONE, HK_NONE, HK_NONE},
    {HK_PgDn, HK_End, HK_F11, HKR_9, HKR_Minus, HKR_8, HKR_7, HK_NONE},
    {HK_Space, HK_B, HK_Colon, HK_N, HK_Equal, HK_Period, HK_6, HK_F8},
    {HK_Z, HK_X, HK_Delete, HK_T, HK_Slash, HK_Comma, HK_5, HK_F7},
    {HK_Pause, HK_Tab, HK_PrtScr, HKR_2, HKR_3, HKR_1, HKR_0, HK_MODE}
};

void update_key_matrix(char row, char col, char onoff)
{
    static uint16_t hid_report[4]={0,0,0,0};
    static char (*hid_keymap)[8]=hid_keymap_dvorak;

    unsigned char key=hid_keymap[row][col];
    unsigned char *report =(unsigned char *)hid_report;
    char i;

    if(key==HK_MODE)
    {
        if(!onoff)
        {
            if(hid_keymap==hid_keymap_dvorak)
            {
                hid_keymap=hid_keymap_qwerty;
                GPIOB->BSRR = (1<<2);
            }
            else
            {
                hid_keymap=hid_keymap_dvorak;
                GPIOB->BRR = (1<<2);
            }
        }
        return;
    }

    if(key>=0x80)   // Alt, Ctrl, Shift
    {
        uint8_t bitset = 1<<(key&7);
        if(onoff)   // non-zero is key up
            report[0] &= (~bitset);
        else
            report[0] |= bitset;
    }
    else
    {
        if(onoff)   // non-zero is key up
        {
            for(i=2;i<8;i++)
            {
                if(report[i]==key)
                {
                    report[i]=0;
                    break;
                }
            }
        }
        else
        {
            for(i=2;i<8;i++)
            {
                if(report[i]==key)
                    break;
                if(report[i]==0)
                {
                    report[i]=key;
                    break;
                }
            }
        }
    }
    for(i=0;i<4;i++)
        USB_PMA[192+i]=hid_report[i];
    USB_PMA[5]=8;   //COUNT1_TX
    if(ep1_wait==0)
    {
        USB->EP1R = USB_EP_TYPE_INTERRUPT|USB_EP_STAT_TX0|1;
        ep1_wait=1;
    }
}
